unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls;

type
  TMainForm = class(TForm)
    lstFiles: TListView;
    lblSearchFor: TLabel;
    txtSearchFor: TEdit;
    lblSearchIn: TLabel;
    txtSearchIn: TEdit;
    btnSearchIn: TSpeedButton;
    pnlStatus: TPanel;
    btnStartStop: TButton;
    procedure btnSearchInClick(Sender: TObject);
    procedure btnStartStopClick(Sender: TObject);
  private
    { Private declarations }
    Searching: Boolean;
    procedure SearchForFiles(Dir: String);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  ShlObj, ActiveX, DateUtils;

procedure TMainForm.btnSearchInClick(Sender: TObject);
var
  Info: TBrowseInfoA;
  Buffer: PChar;
  ShellMalloc: IMalloc;
  ItemIDList: PItemIDList;
  S: String;
begin
  SHGetMalloc(ShellMalloc);
  Buffer := ShellMalloc.Alloc(MAX_PATH);
  Info.hwndOwner := Handle;
  Info.pidlRoot := nil;
  Info.pszDisplayName := Buffer;
  Info.lpszTitle := 'Select folder';
  Info.ulFlags := BIF_RETURNFSANCESTORS;
  Info.lpfn := nil;
  ItemIDList := SHBrowseForFolder(Info);
  if ItemIDList <> nil then
  begin
    SHGetPathFromIDList(ItemIDList, Buffer);
    ShellMalloc.Free(ItemIDList);
  end;
  S := Buffer;
  txtSearchIn.Text := S;
  ShellMalloc.Free(Buffer);
end;

procedure TMainForm.btnStartStopClick(Sender: TObject);
begin
  if Searching then
  begin
    Searching := False;
    btnStartStop.Caption := 'Start';
    txtSearchIn.Enabled := True;
    txtSearchFor.Enabled := True;
    lblSearchIn.Enabled := True;
    lblSearchFor.Enabled := True;
    btnSearchIn.Enabled := True;
    pnlStatus.Caption := 'Idle';
  end
  else
  begin
    Searching := True;
    lstFiles.Clear;
    btnStartStop.Caption := 'Stop';
    txtSearchIn.Text := IncludeTrailingBackslash(txtSearchIn.Text);
    txtSearchIn.Enabled := False;
    txtSearchFor.Enabled := False;
    lblSearchIn.Enabled := False;
    lblSearchFor.Enabled := False;
    btnSearchIn.Enabled := False;
    SearchForFiles(txtSearchIn.Text);
    Searching := False;
    btnStartStop.Caption := 'Start';
    txtSearchIn.Enabled := True;
    txtSearchFor.Enabled := True;
    lblSearchIn.Enabled := True;
    lblSearchFor.Enabled := True;
    btnSearchIn.Enabled := True;
    pnlStatus.Caption := 'Idle';
  end;
end;

procedure TMainForm.SearchForFiles(Dir: String);
var
  sR: TSearchRec;
  FileAttrs: String;
  L: TListItem;
begin
  pnlStatus.Caption := Dir;
  if FindFirst(Dir + txtSearchFor.Text, faAnyFile, sR) = 0 then
  begin
    repeat
      Application.ProcessMessages;
      FileAttrs := '';
      L := lstFiles.Items.Add;
      L.Caption := Dir + sR.Name;
      if (sR.Attr and faDirectory) <> 0 then
        FileAttrs := FileAttrs + 'DIR ';
      if (sR.Attr and faReadOnly) <> 0 then
        FileAttrs := FileAttrs + 'r';
      if (sR.Attr and faHidden) <> 0 then
        FileAttrs := FileAttrs + 'h';
      if (sR.Attr and faSysFile) <> 0 then
        FileAttrs := FileAttrs + 's';
      if (sR.Attr and faVolumeID) <> 0 then
        FileAttrs := FileAttrs + 'v';
      if (sR.Attr and faArchive) <> 0 then
        FileAttrs := FileAttrs + 'a';
      L.SubItems.Add(FileAttrs);
      L.SubItems.Add(IntToStr(sR.Size));
      L.SubItems.Add(DateTimeToStr(FileDateToDateTime(sR.Time)));
    until (FindNext(sR) <> 0) or (not Searching);
    FindClose(sr);
  end;


  if FindFirst(Dir + '*.*', faDirectory, sR) = 0 then
  begin
    repeat
      Application.ProcessMessages;
      if ((sR.Attr and faDirectory) <> 0) and
        (sR.Name[1] <> '.') then
        SearchForFiles(Dir + sR.Name + '\');
    until (FindNext(sR) <> 0) or (not Searching);
    FindClose(sr);
  end;
end;

end.
